@@ -1,9 +1,9 @@
 /*
  * 
- * $Id: field.c,v 1.29.8.1 2001/09/02 07:13:37 rug Exp $
+ * $Id: field.c,v 1.29.8.8 2004/03/20 15:25:09 usu Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2004 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -48,6 +48,13 @@
 #include "replace.h"
 #include "idxname.h"
 
+/*
+ * Private variables
+ */
+
+static int cache_idx = 0, cache_num = 0;
+static struct field_cache fc[FIELD_CACHE_SIZE];
+
 /*
  *
  * Private functions
@@ -58,6 +65,7 @@ static void apply_field_alias ( char *field );
 static int is_field_safe_char ( int c );
 static void make_fullpathname_field ( int n );
 
+
 static void 
 apply_field_alias(char *field)
 {
@@ -130,15 +138,17 @@ nmz_get_field_name(const char *fieldpat)
 {
     static char field_name[BUFSIZE]; /* storing field name */
     char *tmp = field_name;
+    int count = 0;
 
     fieldpat++;  /* ignore beggining '+' mark */
-    while (*fieldpat) {
+    while (*fieldpat && count < BUFSIZE - 1) {
         if (! is_field_safe_char(*fieldpat)) {
             break;
         }
         *tmp = *fieldpat;
         tmp++;
         fieldpat++;
+	count++;
     }
     *tmp = '\0';
 
@@ -149,22 +159,14 @@ nmz_get_field_name(const char *fieldpat)
 void 
 nmz_get_field_data(int idxid, int docid, const char *field, char *data) 
 {
-    char fname[BUFSIZE];
-    char tmpfield[BUFSIZE];
+    char fname[BUFSIZE] = "";
+    char tmpfield[BUFSIZE] = "";
     int i;
-    static int cache_idx = 0, cache_num = 0;
     FILE *fp_field, *fp_field_idx;
-    struct field_cache {
-	int idxid;
-	int docid;
-	char field[BUFSIZE];
-	char data[BUFSIZE];
-    };
-    static struct field_cache fc[FIELD_CACHE_SIZE];
 
     strcpy(data, ""); /* For safety. */
 
-    strcpy(tmpfield, field);
+    strncpy(tmpfield, field, BUFSIZE - 1);
     apply_field_alias(tmpfield);  /* This would overwrite `tmpfield' */
 
     /* 
@@ -177,15 +179,15 @@ nmz_get_field_data(int idxid, int docid, const char *field, char *data)
 	    strcmp(tmpfield, fc[i].field) == 0)
 	{  /* cache hit! */
 	    nmz_debug_printf("field cache [%s] hit!\n", tmpfield);
-	    strcpy(data, fc[i].data);
+	    strncpy(data, fc[i].data, BUFSIZE - 1);	/* data length should be BUFSIZE - 1 */
 	    return;
 	}
     }
 
     /* Make a pathname */
     make_fullpathname_field(idxid);
-    strcpy(fname, NMZ.field);
-    strcat(fname, tmpfield);
+    strncpy(fname, NMZ.field, BUFSIZE - 1);
+    strncat(fname, tmpfield, BUFSIZE - strlen(fname) - 1);
     
     fp_field = fopen(fname, "rb");
     if (fp_field == NULL) {
@@ -193,10 +195,11 @@ nmz_get_field_data(int idxid, int docid, const char *field, char *data)
 	return;
     }
 
-    strcat(fname, ".i");
+    strncat(fname, ".i", BUFSIZE - strlen(fname) - 1);
     fp_field_idx = fopen(fname, "rb");
     if (fp_field_idx == NULL) {
         nmz_warn_printf("%s: %s", fname, strerror(errno));
+        fclose(fp_field);
 	return;
     }
 
@@ -215,12 +218,25 @@ nmz_get_field_data(int idxid, int docid, const char *field, char *data)
     /* Cache */
     fc[cache_idx].idxid = idxid;
     fc[cache_idx].docid = docid;
-    strcpy(fc[cache_idx].field, tmpfield);
-    strcpy(fc[cache_idx].data, data);
+    strncpy(fc[cache_idx].field, tmpfield, BUFSIZE - 1);
+    fc[cache_idx].field[BUFSIZE - 1] = '\0';
+    strncpy(fc[cache_idx].data, data, BUFSIZE - 1);
+    fc[cache_idx].data[BUFSIZE - 1] = '\0';
     cache_idx = (cache_idx + 1) % FIELD_CACHE_SIZE;
     if (cache_num < FIELD_CACHE_SIZE) {
 	cache_num++;
     }
 }
 
-
+void
+nmz_free_field_cache(void)
+{
+       int i;
+       for(i = 0; i < cache_num; i++) {
+               fc[i].idxid =0;
+               fc[i].docid =0;
+               strcpy(fc[i].field, "");
+               strcpy(fc[i].data, "");
+       }
+       cache_num = 0;
+}
