@@ -52,5 +52,5 @@
 char *
 nmz_get_version(void)
 {
-    return((char *) "2.0.13RC2");
+    return((char *) "2.0.14");
 }
