@@ -1,9 +1,9 @@
 /*
  * 
- * $Id: util.c,v 1.75.4.10 2002/09/03 04:58:00 knok Exp $
+ * $Id: util.c,v 1.75.4.14 2004/03/20 15:25:10 usu Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000,2001 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2004 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -457,12 +457,15 @@ nmz_readfile(const char *fname)
     buf = malloc(fstatus.st_size + 1);
     if (buf == NULL) {
 	nmz_set_dyingmsg(nmz_msg("%s: %s", fname, strerror(errno)));
+        fclose(fp);
 	return NULL;
     }
     if (fstatus.st_size != 0 &&
 	fread(buf, sizeof(char), fstatus.st_size, fp) == 0) 
     {
         nmz_set_dyingmsg(nmz_msg("%s: %s", fname, strerror(errno)));
+        free(buf);
+        fclose(fp);
 	return NULL;
     }
     *(buf + fstatus.st_size) = '\0';
@@ -489,8 +492,10 @@ nmz_decode_uri(char *str)
     int i, j;
     for (i = j = 0; str[i]; i++, j++) {
 	if (str[i] == '%') {
-	    str[j] = decode_uri_sub(str[i + 1], str[i + 2]);
-	    i += 2;
+	    if (str[i+1] != 0 && str[i+2] != 0) {
+		str[j] = decode_uri_sub(str[i + 1], str[i + 2]);
+		i += 2;
+	    }
 	} else if (str[i] == '+') {
 	    str[j] = ' ';
 	} else {
