@@ -1,6 +1,6 @@
 #
 # -*- Perl -*-
-# $Id: codeconv.pl,v 1.11.8.2 2001/07/09 18:14:35 takesako Exp $
+# $Id: codeconv.pl,v 1.11.8.6 2004/01/30 15:15:38 usu Exp $
 # Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
 # Copyright (C) 2000 Namazu Project All rights reserved.
 #     This is free software with ABSOLUTELY NO WARRANTY.
@@ -134,7 +134,7 @@ sub toeuc ($) {
 	my $nkf_opt = "-emXZ1";
 
 	if ($var::USE_NKF_MODULE) {
-	    $$contref = NKF::nkf($nkf_opt, $$contref);
+	    $$contref = NKF::nkf($nkf_opt, $$contref); # namazu-devel-ja #3152 -> backed out
 	} else {
 	    my $nkftmp = util::tmpnam("NMZ.nkf");
 	    {
@@ -149,4 +149,24 @@ sub toeuc ($) {
 	}
     }
 }
+
+sub eucjp_zen2han_ascii ($) {
+    my ($str) = @_;
+    if (util::islang("ja")) {
+	$str =~ s/([\xa1-\xfe][\xa1-\xfe]|\x8e[\xa1-\xdf]|\x8f[\xa1-\xfe][\xa1-\xfe])/
+	my $tmp = $1;
+	$tmp =~ m!\xa3([\xb0-\xb9\xc1-\xda\xe1-\xfa])! ? $1 & "\x7F" : $tmp/gse;
+    }
+    $str;
+}
+
+sub normalize_eucjp ($) {
+    my ($contref) = @_;
+    if (util::islang("ja")) {
+        $$contref = codeconv::eucjp_han2zen_kana($$contref);
+        $$contref = codeconv::eucjp_zen2han_ascii($$contref);
+    }
+    $contref;
+}
+
 1;
